@@ -1,3 +1,5 @@
+#!/usr/bin/python
+
 '''
 LIBrary for YARE (Yet Another Regular Expression) pattern match
 
@@ -18,10 +20,14 @@ CONTENT
 
 1. INSTALLATION
 
+From terminal type:
+
     $ pip3 install libyare
 
 2. HELP
 
+From terminal type:
+
     $ pydoc libyare
 
 or, from a Python IDE:
@@ -31,7 +37,7 @@ or, from a Python IDE:
 
 3. DEFINITIONS
 
-"YARE"  (Yet  Another  Regular Expression) is a regular expression format aimed to be more
+"YARE"  (Yet  Another Regular Expression) is a regular expression format aimed to be more
 readable (even at the cost of being less powerful) than standard regular expressions.
 
 A "pattern" can be:
@@ -44,7 +50,7 @@ A "pattern" can be:
         • a "numeric pattern", made by:
             • a comparison operator '=' '>' '<' '>=' '<=' or '<>'
             • an integer (or TGMK, see later) literal
-    • a  "compound  pattern",  made combining simple patterns by logical operators '^' '&'
+    • a  "compound  pattern", made combining simple patterns by logical operators '^' '&'
       and ',' and by parentheses '(' and ')'
 
 "Metacharacters" are:
@@ -53,12 +59,12 @@ A "pattern" can be:
     • '!' and '-', used in string patterns, between '[' and ']'
     • '`' backquote, used at beginning of charset patterns
     • '^' (not) '&' (and) and ',' (or) logical operators in compound patterns
-    • '('  and  ')'  parenthesis altering precedence between logical operators in compound
+    • '('  and  ')' parenthesis altering precedence between logical operators in compound
       patterns
 
 4. STRING PATTERNS
 
-A  "string  pattern" is a classic Unix shell style patterns, without extensions introduced
+A  "string pattern" is a classic Unix shell style patterns, without extensions introduced
 by bash shell, so, for instance, curly braces '{' and '}' have no special meaning.
 
     • pattern '*' matches any string
@@ -67,24 +73,24 @@ by bash shell, so, for instance, curly braces '{' and '}' have no special meanin
     • pattern '[!a]' matches any single char except 'a' or 'b'
     • pattern '[a-z]' matches any single char between 'a' and 'z'
     • pattern '[!a-z]' matches any single char not between 'a' and 'z'
-    • pattern  '[a-z0-9_]'  matches any single char between 'a' and 'z' or between '0' and
+    • pattern  '[a-z0-9_]' matches any single char between 'a' and 'z' or between '0' and
       '9' or equal to '_'
-    • pattern '[!a-z0-9_]' matches any single char not between 'a' and 'z' and not between
-      '0' and '9' and not equal to '_'
+    • pattern  '[!a-z0-9_]'  matches  any  single  char  not  between 'a' and 'z' and not
+      between '0' and '9' and not equal to '_'
 
 For further details, look for instance at the Python3 documentation about fnmatch module.
 
-If  a  metacharacter  must belong to a simple string pattern then it must be quoted by '['
+If  a  metacharacter must belong to a simple string pattern then it must be quoted by '['
 and ']', more exactly:
 
     • '*' '?' '[' '^' '&' ',' '(' and ')' must always be quoted
-    • '!'  and '-' if not between '[' and ']' have no special meaning and don't need to be
+    • '!' and '-' if not between '[' and ']' have no special meaning and don't need to be
       quoted
-    • '`'  needs  to  be quoted only in first position inside the simple string pattern in
+    • '`'  needs  to be quoted only in first position inside the simple string pattern in
       order to avoid confusion with charset pattern
-    • '<'  '='  and  '>' need to be quoted only in first position inside the simple string
+    • '<'  '='  and '>' need to be quoted only in first position inside the simple string
       pattern in order to avoid confusion with numeric pattern
-    • ']'  only can't be quoted, but you shouldn't need it because an unmatched ']' has no
+    • ']' only can't be quoted, but you shouldn't need it because an unmatched ']' has no
       special meaning and doesn't raise a syntax error, as unmatched '[' '(' and ')' do
 
 Some example:
@@ -92,7 +98,7 @@ Some example:
     • pattern '[(]*[)]' matches any string starting with '(' and ending with ')'
     • pattern '[[]*]' matches any string starting with '[' and ending with ']'
     • patterns '[`]*`' and '[`]*[`]' both match any string starting and ending with '`'
-    • patterns  '[<]*>'  and  '[<]*[>]' both match any string starting with '<' and ending
+    • patterns  '[<]*>'  and '[<]*[>]' both match any string starting with '<' and ending
       with '>'
 
 You can quote metacharacter '!' too, but not immediately after '['.
@@ -100,7 +106,7 @@ You can quote metacharacter '!' too, but not immediately after '['.
     • pattern '[?!]' matches '?' and '!' only
     • pattern '[!?]' matches any character except '?'
 
-You  can  quote  '-'  too,  but immediately after '[' or before ']' only, otherwise it's a
+You  can  quote  '-'  too, but immediately after '[' or before ']' only, otherwise it's a
 metacharacter denoting a character interval:
 
     • patterns '[-pr]' and '[pr-]' match '-' 'p' and 'r'
@@ -115,17 +121,17 @@ Descending character intervals are not allowed:
 
     • pattern '[z-a]' raises a syntax error
 
-They  are  only  two  differences  between  patterns  defined  by  fnmatch()/fnmatchcase()
+They  are  only  two  differences  between  patterns  defined  by fnmatch()/fnmatchcase()
 functions in Python3 fnmatch module and patterns accepted by YARE:
 
-    • unmatched  '['  (as  in  pattern 'abc[def') is allowed by fnmatch but is rejected by
+    • unmatched  '['  (as  in pattern 'abc[def') is allowed by fnmatch but is rejected by
       YARE as a syntax error
-    • null pattern '' is allowed by fnmatch but is rejected by YARE as a syntax error (see
-      in  paragraph  5. COMPOUND PATTERNS for a workaround to match a null string by a not
-      null pattern)
+    • null  pattern  ''  is  allowed by fnmatch but is rejected by YARE as a syntax error
+      (see in paragraph 5. COMPOUND PATTERNS for a workaround to match a null string by a
+      not null pattern)
 
-Simple string patterns can be case-sensitive or case-insensitive, depending on which match
-function you use, see paragraph 6. THREE FLAVOURS.
+Simple  string  patterns  can  be  case-sensitive or case-insensitive, depending on which
+match function you use, see paragraph 6. THREE FLAVOURS.
 
 5. CHARSET PATTERNS
 
@@ -134,13 +140,13 @@ A "charset pattern" contains:
     • a backquote '`'
     • a single-char classic Unix shell style pattern
 
-A  charset  pattern matches a string if all chars in string match single-char classic Unix
+A  charset pattern matches a string if all chars in string match single-char classic Unix
 shell style pattern.
 
 Example:
 
-    • '`[a-z0-9]' matches any string made of zero or more underscore alphabetic or numeric
-      characters
+    • '`[a-z0-9]'  matches  any  string  made  of  zero  or more underscore alphabetic or
+      numeric characters
 
 Charset patterns are always case-sensitive.
 
@@ -160,7 +166,7 @@ No other comparison operator (as '==' '!=' '><' '!<' or '!>') is allowed.
     • pattern '>x' matches any string whose numeric value is greater than x
     • pattern '>=x' matches any string whose numeric value is greater than or equal to x
 
-where  x  is  any  integer  literal (or TGMK literal, see next paragraph), so for instance
+where  x  is  any  integer literal (or TGMK literal, see next paragraph), so for instance
 patterns '>=1024&<2048' and '>=1k&<2k' are equivalent and both match:
 
     • '1024', '1025' ... '2047', but also:
@@ -170,7 +176,7 @@ Numeric patterns are always case-insensitive,
 
 7. TGMK LITERALS
 
-"TGMK"  (Tebi-Gibi-Mebi-Kibi)  is  a  human-readable  lossless  case-insensitive 1024-base
+"TGMK"  (Tebi-Gibi-Mebi-Kibi)  is  a  human-readable  lossless case-insensitive 1024-base
 integer representation, suitable for bits and bytes.
 
 A "TGMK literal" is a string containing:
@@ -178,7 +184,7 @@ A "TGMK literal" is a string containing:
     • zero or more leading blanks
     • an optional ('+' or '-') "sign"
     • one or more "1024-base-digits", each made by:
-        • a  "mantissa",  a string of one or more decimal digits, representing an unsigned
+        • a  "mantissa", a string of one or more decimal digits, representing an unsigned
           decimal integer constant, as '0', '1' or '100000'.
         • a "characteristic", a letter in 'KMGTPEZY' (or 'kmgtpezy', case has no meaning)
     • zero or more trailing blanks
@@ -219,7 +225,7 @@ TGMK literals are always case-insensitive, so YARE numeric patterns are too.
 
 In the following examples, p and q are two simple patterns.
 
-A  "compound  pattern"  is built aggregating simple patterns by logic operators '^' (not),
+A  "compound  pattern" is built aggregating simple patterns by logic operators '^' (not),
 '&' (and) and ',' (or) and parenthesis '(' and ')':
 
     • pattern '^p' matches any string not matching pattern p
@@ -232,24 +238,24 @@ Two '^' characters cancel each other out:
 
 Precedence is of course '^' > '&' > ',':
 
-    • pattern  '??,^s*&*t' matches any 2-chars string, or any string not starting with 's'
+    • pattern '??,^s*&*t' matches any 2-chars string, or any string not starting with 's'
       and ending with 't'
-    • pattern '>=10&<13,>20&<=23' matches any int (or TGMK) string worth 10 11 12 21 22 or
-      23
+    • pattern  '>=10&<13,>20&<=23'  matches any int (or TGMK) string worth 10 11 12 21 22
+      or 23
 
-But  precedence  can be forced by parenthesis '(' and ')': so for instance, let p and q be
+But  precedence can be forced by parenthesis '(' and ')': so for instance, let p and q be
 two simple patterns, then De Morgan's laws tell us that:
 
     • patterns '^p&^q' and '^(p,q)' are equivalent
     • patterns '^p,^q' and '^(p&q)' are equivalent
 
-As  said before, a null pattern '' is not allowed, but if you need to match a null string,
+As said before, a null pattern '' is not allowed, but if you need to match a null string,
 then you can use a workaround:
 
     • pattern '?*' matches everything, except the null string, so...
     • pattern '^?*' matches the null string, and nothing else
 
-Be  careful  using  '^'  operator  with  charset patterns, for instance the following four
+Be  careful  using  '^'  operator  with charset patterns, for instance the following four
 patterns have four distict meanings:
 
     • '`[0-9]' matches strings containing numeric chars only, hence containing:
@@ -273,13 +279,13 @@ YARE match comes in three flavours:
     • cimatch(string, pattern, case='i') # case-insensitive match
     • cxmatch(string, pattern, case='d') # case-mixed match
 
-Of  course  csmatch(string,  pattern, case='i') is equivalent to cimatch(string, pattern),
+Of  course  csmatch(string, pattern, case='i') is equivalent to cimatch(string, pattern),
 and so on.
 
-In  case-mixed  match any simple string pattern can be case-sensitive or case-insensitive,
+In  case-mixed match any simple string pattern can be case-sensitive or case-insensitive,
 depending on its content:
 
-    • if  the  simple  pattern  contains  one  or  more  lowercase  letters  then match is
+    • if  the  simple  pattern  contains  one  or  more  lowercase  letters then match is
       case-sensitive
     • otherwise the match is case-insensitive
 
@@ -287,10 +293,10 @@ Example:
 
     • pattern 'RAM,?*.db' matches 'ram', 'Ram', 'RAM' or 'xyz.db', but not 'xyz.Db'
 
-A  lowercase letter is any letter having a distinct corresponding uppercase letter, so for
+A lowercase letter is any letter having a distinct corresponding uppercase letter, so for
 instance 'å' is said to be a lowercase letter because 'å'.upper() == 'Å' != 'å'.
 
-The  three  flavors  only have to do with string patterns because, as said before, charset
+The  three  flavors only have to do with string patterns because, as said before, charset
 patterns are always case-sensitive, and numeric patterns are always case-insensitive.
 
 10. EXCEPTIONS
@@ -299,8 +305,8 @@ patterns are always case-sensitive, and numeric patterns are always case-insensi
     • ValueError if an incorrect TGMK literal is found in pattern
     • ValueError if case is not 's' 'i' or 'x'
 
-Note that if an incorrect TGMK literal is found in string to be matched, then no exception
-is raised, but the match returns with a result of False, for example:
+Note  that  if  an  incorrect  TGMK  literal  is  found  in string to be matched, then no
+exception is raised, but the match returns with a result of False, for example:
 
     >>> csmatch('3.14', '>3&<4')
     False
@@ -309,6 +315,9 @@ because '3.14' contains a dot, so it's not a correct TGMK literal.
 
 11. HISTORY
 
+    • 0.4.3
+        • updated: documentation
+
     • 0.4.2
         • updated: documentation
 
@@ -324,8 +333,9 @@ from re import error as ReError
 
 #----- constants -----
 
-__version__ = '0.4.2'
-__all__ = ['csmatch','cimatch','cxmatch']
+__version__ = '0.4.3'
+
+__all__ = ['csmatch', 'cimatch', 'cxmatch']
 
 PATTERN, LEFT, RIGHT, OR, AND, NOT = '*(),&^' # tokens: patterns and operators
 
@@ -354,7 +364,7 @@ def csmatch(string, pattern, case='s'):
 
     def scanner(pattern):
         '''
-translate  {pattern}  into  a  list of {tokens}, checking the correct token sequence, each
+translate  {pattern}  into  a list of {tokens}, checking the correct token sequence, each
 token will be an operator or a simple (string or charset or numeric) pattern
 '''
         not_in_simple, in_simple, in_brakes = [0, 1, 2] # scanner status values
@@ -417,10 +427,13 @@ token will be an operator or a simple (string or charset or numeric) pattern
             return fnmatchcase(string, pattern)
         elif case == 'i': # case insensitive
             return fnmatchcase(string.upper(), pattern.upper())
-        elif pattern != pattern.upper(): # case mixed, if (pattern) contain lowercases...
-            return fnmatchcase(string, pattern) # ...then case sensitive...
+        elif case == 'x': # case mixed
+            if pattern != pattern.upper(): # if (pattern) contain lowercases...
+                return fnmatchcase(string, pattern) # ...then case sensitive...
+            else:
+                return fnmatchcase(string.upper(), pattern) # ...else case insensitive
         else:
-            return fnmatchcase(string.upper(), pattern) # ...else case insensitive
+            raise ValueError("case must be 's' 'i' or 'x'")
 
     def apply(operator):
         'apply {operator} on {values}'
